c **********************************************************************
c *   Arrangement (Input of parameters & Initialization)               *
c *                                                                    *
c *   Input variables:                                                 *
c *     nrun          : Run number (one or two figures except zero)    *
c *                   : If it is negative, simulation will be made in  *
c *                   : the restart mode of the relevant run number.   *
c *     nstep         : Number of time steps for this run              *
c *                   : nstep * dt = Simulation time                   *
c *     iout          : Output of the initial distributions            *
c *                   : = 0, NO; otherwise, YES                        *
c *     jout          : Output of the simulated distributions          *
c *                   : = 0, only the last step (recommended);         *
c *                   : otherwise, every jout steps                    *
c *     lout          : Print of the divergence for verification       *
c *                   : = 0, NO; otherwise, every lout steps           *
c *                                                                    *
c *   # In the restart mode, the parameters below are automatically    *
c *     set to those in the former run.                                *
c *                                                                    *
c *     isgs          : SGS model                                      *
c *                   : = 0, Standard; otherwise, Sullivan et al.      *
c *     isfc          : Lower boundary condition                       *
c *                   : > 2, Prescript (C_drag will be computed for    *
c *                   :         temporal-mean velocity in each grid);  *
c *                   : = 2, Prescript (C_drag will be computed for    *
c *                   :         instantaneous velocity in each grid);  *
c *                   : = 1, Prescript (C_drag will be computed for    *
c *                   :         horizontal-mean velocity);             *
c *                   : = 0, Forecast; < 0, Stress free                *
c *     irad          : Radiation                                      *
c *                   : = 0, OFF; otherwise, ON                        *
c *     icon          : Condensation                                   *
c *                   : = 0, OFF; > 0, ON;                             *
c *                   : < 0, Specific humidity also is not predicted.  *
c *     isca          : Diffusion of a scalar quantity                 *
c *                   : = 0, OFF; otherwise, ON                        *
c *                                                                    *
c *     jmin          : Initial time starting from 0000 LST  (minute)  *
c *     rlat          : Latitude                             (degree)  *
c *                   : > 90.0, No Coriolis force                      *
c *     dt            : Time step                               (sec)  *
c *                   : = roughly 0.1 * dx / (Geostrophic wind)        *
c *     dx, dy, dz    : Grid spacings                             (m)  *
c *                                                                    *
c *   # Numbers of grids (nx, ny, nz) are specified in Declare.h.      *
c *                                                                    *
c *     ugsf, vgsf    : Geostrophic wind at the surface         (m/s)  *
c *     aglw, agmd    : Height of the top of the lower layer and that  *
c *                   : of the middle layer (for ug and vg)       (m)  *
c *     gulw, gvlw    : Vertical shear of ug and vg in the lower       *
c *                   : layer                                 (m/s m)  *
c *                   : If not specified, they are the same as those   *
c *                   : in the middle layer.                           *
c *     gumd, gvmd    : Those in the middle layer             (m/s m)  *
c *                   : If not specified, they are equal to 0.0.       *
c *                                                                    *
c *   # In a barotropic fluid, aglw, ..., gvmd need not be specified.  *
c *     However, if aglw < 0, the initial velocity distribution is     *
c *     given as the Ekman spiral with the PBL top of abs(aglw).       *
c *                                                                    *
c *     tref          : Reference potential temperature           (K)  *
c *                   : If not specified, it is set to the mean value  *
c *                   : in the computational domain.                   *
c *     tsf, qsf, psf : Potential temperature, specific humidity, and  *
c *                   : pressure at the surface       (K, kg/kg, hPa)  *
c *     hglw, hgmd    : Height of the top of the lower layer and that  *
c *                   : of the middle layer (for t and q)         (m)  *
c *     gtlw, gqlw    : Vertical gradients of t and q in the lower     *
c *                   : layer                          (K/m, kg/kg m)  *
c *                   : If not specified, they are the same as those   *
c *                   : in the middle layer.                           *
c *     gtmd, gqmd    : Those in the middle layer      (K/m, kg/kg m)  *
c *                   : If not specified, they are the same as those   *
c *                   : in the upper layer.                            *
c *     gamt, gamq    : Those in the upper layer       (K/m, kg/kg m)  *
c *                                                                    *
c *   # If tsf =< 0.0, the vertical distributions are given through    *
c *     the file OBS and tref, ..., gqmd need not be specified.        *
c *     However, gamt and gamq must be specified.                      *
c *                                                                    *
c *     z0m, z0h      : Roughness length for momentum and heat    (m)  *
c *     tfmx, qfmx    : Fluxes of heat and vapor at noon (K m/s, m/s)  *
c *     dhour         : Length of the daytime                  (hour)  *
c *                   : tfmx, qfmx, dhour are used only in isfc > 0.   *
c *                   : If dhour=<0, fluxes are fixed at tfmx & qfmx.  *
c *     tdeep         : Deep soil ABSOLUTE temperature            (K)  *
c *     swet          : Soil wetness                        (0.0-1.0)  *
c *     crso          : Soil heat capacity                  (J/K m^3)  *
c *     fkso          : Soil thermal conductivity             (W/K m)  *
c *                   : If either crso or fkso is less than zero,      *
c *                   : both of them will be computed from swet.       *
c *                   : tdeep, ..., fkso are used only in isfc = 0.    *
c *                                                                    *
c *     elat          : Declination                          (degree)  *
c *     albd          : Albedo for the solar radiation      (0.0-1.0)  *
c *     clov, ulov    : Cloud fraction and LWP over the top boundary   *
c *                   :                              (0.0-1.0, g/m^2)  *
c *                   : These are used only in sub.radiation.          *
c *                                                                    *
c *     nflay         : Number of layers to which fluctuations are     *
c *                   : added (Normally about nz/3)                    *
c *     frate         : Rate of fluctuation intensity to the geo-      *
c *                   : strophic wind (Normally about 5-6%)            *
c *                   : These are used only in the initialization of   *
c *                   : the flow fields.                               *
c *                                                                    *
c *     nspl, nspu    : Number of the sponge layers near the bottom    *
c *                   : and top; = 0, no sponge layer is placed.       *
c *     csp           : Rayleigh damping coefficient at the boundary   *
c *                                                                    *
c *     nstot         : Total number of time steps                     *
c *                   : DO NOT touch this.                             *
c *                                                                    *
c *   * Added on April 13, 2006 *                                      *
c *                                                                    *
c *     xbub, ...zbub : Coordinates at the center of the bubble   (m)  *
c *     xbr           : Radius of the bubble in the x direction   (m)  *
c *     ybr, zbr      : Same as above but for the y and z direction    *
c *     tbub          : Potential temp. at the center of the bubble    *
c *                   : For the present, this value is given to all    *
c *                   : grids within the bubble                   (K)  *
c **********************************************************************
c
      block data constants
c
      include  "Declare.h"
c
      data       g,     rd,     cp,     p0,     t0,     ev
     :    /   9.81,  287.0, 1004.0, 1000.0, 273.15,  2.5e6/
      data    csgs,    ctk,    tpr,    anu,    tnu
     :    /   0.18,    0.1,.333333, 1.5e-5, 2.1e-5/
      data     gmm,    gmh,    aph,    spr,     vk
     :    /   16.0,   16.0,    5.0,   1.00,   0.40/
c  ***  In Businger et al. (1971)  ***
c     data     gmm,    gmh,    aph,    spr,     vk
c    :    /   15.0,    9.0,    4.7,   0.74,   0.35/
c
      end
c **********************************************************************
c
      subroutine  arrangement (
     :                 nssta, nstot,  jout,  lout,
     :                  isgs,  isfc,  irad,  icon,  isca,
     :                  jsec,  rlat,
     :                   z0m,   z0h,  tfmx,  qfmx, dhour,
     :                 tdeep,  swet,  crso,  fkso,
     :                  elat,  albd,  clov,  ulov,
     :                  nspl,  nspu,   csp )
c
      include  "Declare.h"
      include  "IOfile.h"
c
      character (len=5) :: f_out, f_rst
c
      namelist  /les1/  nrun, nstep,  iout,  jout,  lout
      namelist  /les2/  isgs,  isfc,  irad,  icon,  isca,
     :                  jmin,  rlat,    dt,    dx,    dy,    dz,
     :                         ugsf,  vgsf,         aglw,  agmd,
     :                  gulw,  gvlw,  gumd,  gvmd,
     :                  tref,   tsf,   qsf,   psf,  hglw,  hgmd,
     :                  gtlw,  gqlw,  gtmd,  gqmd,  gamt,  gamq,
     :                   z0m,   z0h,  tfmx,  qfmx, dhour,
     :                 tdeep,  swet,  crso,  fkso,
     :                  elat,  albd,  clov,  ulov,
     :                 nflay, frate,
     :                  nspl,  nspu,   csp, nstot
      namelist  /les3/  xbub,  ybub,  zbub,   xbr,   ybr,   zbr,  tbub
c
c  ***  Default values  ***
      nrun  =    1
      nstep =18000
      iout  =    0
      jout  =    0
      lout  =    0
c
      isgs  =    0
      isfc  =    1
      irad  =    0
      icon  =    0
      isca  =    0
c
      jmin  =  360
      rlat  =   35.0
      dt    =    0.2
      dx    =   20.0
      dy    =   20.0
      dz    =   20.0
c
      ugsf  =   10.0
      vgsf  =    0.0
      aglw  =    0.0
      agmd  =    0.0
      gulw  =  -99.0
      gvlw  =  -99.0
      gumd  =  -99.0
      gvmd  =  -99.0
c
      tref  =    0.0
      tsf   =  300.0
      qsf   =    0.0
      psf   = 1000.0
      hglw  =    0.0
      hgmd  =    0.0
      gtlw  =  -99.0
      gqlw  =  -99.0
      gtmd  =  -99.0
      gqmd  =  -99.0
      gamt  =    0.004
      gamq  =    0.0
c
      z0m   =    0.05
      z0h   =    0.0
      tfmx  =    0.05
      qfmx  =    0.0
      dhour =    0.0
      tdeep = tsf
      swet  =    0.2
      crso  =    2.0e6
      fkso  =    0.4
c
      elat  =    0.0
      albd  =    0.2
      clov  =    0.0
      ulov  =    0.0
c
      nflay = nz/3
      frate =    0.06
c
      nspl  =    0
      nspu  =    0
      csp   =    0.005
c
      xbub  =  -99.9
      ybub  =  -99.9
      zbub  =  -99.9
      xbr   =    0.0
      ybr   =    0.0
      zbr   =    0.0
      tbub  =    0.0
c  ************************
c
      read ( 5,nml=les1)
c
      if ( jout .le. 0 ) jout = 2**30
      if ( lout .le. 0 ) lout = 2**30
c
      if     ( nrun .eq. 0 .or. abs(nrun) .gt. 99 ) then
        stop 'Run number error: ABS(nrun) must be from 1 to 99!'
c
      elseif ( nrun .gt. 0 ) then
        write(f_out,'(i2.2,a3)'      )  abs(nrun), '.01'
c
        read ( 5,nml=les2)
c
c  ***
        if     ( isfc .eq. 0 ) then
          if   ( qsf  .le. 0.0 ) stop 'qsf must be larger than zero!'
            irad = 1
        elseif ( isfc .gt. 0 ) then
          if   ( qsf  .le. 0.0 .and. tsf .gt. 0.0 ) then
            irad = 0
            if ( qfmx .le. 0.0 ) icon = -1
          end if
c
          if   ( icon .lt. 0   ) qfmx = 0.0
        else
            irad = 0
        end if
c  ***
                               hgup = dz * nz
        if ( agmd .gt.  hgup ) agmd = hgup
        if ( aglw .gt.  agmd ) aglw = agmd
        if ( hgmd .gt.  hgup ) hgmd = hgup
        if ( hglw .gt.  hgmd ) hglw = hgmd
c
        if ( gumd .lt. -90.0 ) gumd = 0.0
        if ( gvmd .lt. -90.0 ) gvmd = 0.0
        if ( gulw .lt. -90.0 ) gulw = gumd
        if ( gvlw .lt. -90.0 ) gvlw = gvmd
c
        if ( gtmd .lt. -90.0 ) gtmd = gamt
        if ( gqmd .lt. -90.0 ) gqmd = gamq
        if ( gtlw .lt. -90.0 ) gtlw = gtmd
        if ( gqlw .lt. -90.0 ) gqlw = gqmd
c
        if ( tref .le.   0.0 ) then
          tref = tsf +gtlw*hglw +gtmd*( hgmd    -hglw )
     :                          +gamt*( hgup*0.5-hgmd )
        end if
c
        nssta = 1
        nstot = nstep
        jsec  = jmin*60
c
      else
        write(f_rst(1:3),'(i2.2,a1)')  abs(nrun), '.'
c
        do i = 99,1,-1
          write(f_rst(4:5),'(i2.2)')  i
          open (10,file='para'//f_rst,status='old',iostat=ios)
          if ( ios .eq. 0 ) goto 100
        end do
c
        write(6,*) 'File para'//f_rst(1:3)//'.** does not exist!'
        stop
c
  100   write(f_out,'(i2.2,a1,i2.2)' )  abs(nrun), '.', i+1
c
        read (10,nml=les2)
        close(10)
c
        nssta = nstot +1
        nstot = nstot +nstep
        jsec  = jmin*60
c
      end if
c
      tref = nint( tref*100.0 ) / 100.0
      tsf  = nint( tsf *100.0 ) / 100.0
c
c  *******  The other constants  *******
      if ( abs(rlat) .gt. 90.0 ) then
        fs   = 0.0
        fc   = 0.0
      else
        tlat = rlat * 3.141593/180.0
        fs   = 2.0*7.292e-5*sin( tlat )
        fc   = 2.0*7.292e-5*cos( tlat )
      end if
c
      if ( z0m .gt. 0.0 ) then
        gno = 1.0e6/(1.0e8)**(2.0/3.0)
      else
        gno = 1.0e6/(0.5e8)**(2.0/3.0)
      end if
        gpw = 5.0/3.0
c  *************************************
c
      read ( 5,nml=les3)
c
      call data_selection ( isgs, isfc, irad, icon, isca )
c
      call set_to_zero
      call basic_distribution (
     :                  ugsf,  vgsf,  aglw,  agmd,
     :                  gulw,  gvlw,  gumd,  gvmd,
     :                   tsf,   qsf,  hglw,  hgmd,
     :                  gtlw,  gqlw,  gtmd,  gqmd )
c
c  ***  The basic distributions are replaced by observations.  ***
      if ( tsf .le. 0.0 ) call initial_input ( tsf, psf )
c
      if     ( nrun .gt. 0 ) then
        call initialization  ( ugsf, vgsf, tsf, psf )
c
c  ***  Addition of a bubble  ***
        call add_bubble      ( xbub, ybub, zbub, xbr, ybr, zbr, tbub )
c  ***  Addition of fluctuations  ***
        call add_fluctuation ( isfc, nflay, frate, ugsf, vgsf )
c
        if ( iout .ne. 0 ) then
          call data_open ( f_out(1:3)//'00' )
          call data_output
          call data_close
        end if
c
      else
        call data_restart   ( f_rst )
        call exner_pressure ( psf, ugsf, vgsf )
c
      end if
c
      call data_open ( f_out )
c
      write( 6,nml=les1)
      write( 6,nml=les2)
c
      open (11,file='para'//f_out)
      write(11,nml=les1)
      write(11,nml=les2)
      close(11)
c
      return
      end
c
      subroutine  basic_distribution (
     :                  ugsf,  vgsf,  aglw,  agmd,
     :                  gulw,  gvlw,  gumd,  gvmd,
     :                   tsf,   qsf,  hglw,  hgmd,
     :                  gtlw,  gqlw,  gtmd,  gqmd )
c
      include  "Declare.h"
c
      elav = ( dx*dy*dz )**( 1.0/3.0 )
c
      do j = 1,ny
      do k = 1,mz
      do i = 1,mx
        el(i,k,j) = elav
      end do
      end do
      end do
c
      uulw = ugsf + gulw*  aglw
      uumd = uulw + gumd*( agmd-aglw )
      vvlw = vgsf + gvlw*  aglw
      vvmd = vvlw + gvmd*( agmd-aglw )
      ttlw = tsf  + gtlw*  hglw
      ttmd = ttlw + gtmd*( hgmd-hglw )
      qqlw = qsf  + gqlw*  hglw
      qqmd = qqlw + gqmd*( hgmd-hglw )
c
      do k = 1,mz
        zz = dz*( k-0.5 )
c
        if ( aglw .ge. 0.0 ) then
          if     ( zz .lt. aglw ) then
            ui(k) = ugsf +gulw*  zz
            vi(k) = vgsf +gvlw*  zz
          elseif ( zz .lt. agmd ) then
            ui(k) = uulw +gumd*( zz-aglw )
            vi(k) = vvlw +gvmd*( zz-aglw )
          else
            ui(k) = uumd
            vi(k) = vvmd
          end if
            ug(k) = ui(k)
            vg(k) = vi(k)
c
        else
c  ***  Ekman spiral  ***
            zd = 3.141593*zz / abs(aglw)
            aa = 1.0 -exp(-zd)*cos(zd)
            bb =      exp(-zd)*sin(zd)
c
            ui(k) = ugsf*aa -vgsf*bb
            vi(k) = vgsf*aa +ugsf*bb
            ug(k) = ugsf
            vg(k) = vgsf
        end if
c
        if     ( zz .lt. hglw ) then
            ti(k) =      tsf  +gtlw*  zz        -tref
            qi(k) = max( qsf  +gqlw*  zz       , 0.0 )
        elseif ( zz .lt. hgmd ) then
            ti(k) =      ttlw +gtmd*( zz-hglw ) -tref
            qi(k) = max( qqlw +gqmd*( zz-hglw ), 0.0 )
        else
            ti(k) =      ttmd +gamt*( zz-hgmd ) -tref
            qi(k) = max( qqmd +gamq*( zz-hgmd ), 0.0 )
        end if
      end do
c
      ug(0) = ug(1) -gulw*dz
      vg(0) = vg(1) -gvlw*dz
c
      return
      end
c
      subroutine  set_to_zero
c
      include  "Declare.h"
c
      do j = 1,my
      do k = 1,mz
      do i = 1,mx
        u  (i,k,j) = 0.0;  v  (i,k,j) = 0.0;  w  (i,k,j) = 0.0
        t  (i,k,j) = 0.0;  q  (i,k,j) = 0.0;  ql (i,k,j) = 0.0
        pd (i,k,j) = 0.0;  tk (i,k,j) = 0.0
        fm (i,k,j) = 0.0;  fh (i,k,j) = 0.0
        vt (i,k,j) = 0.0;  vq (i,k,j) = 0.0;  cld(i,k,j) = 0.0
        rhr(i,k,j) = 0.0;  rfx(i,k,j) = 0.0
        sa (i,k,j) = 0.0
        el (i,k,j) = 0.0;  bv (i,k,j) = 0.0;  wo (i,k,j) = 0.0
        uu (i,k,j) = 0.0;  vv (i,k,j) = 0.0;  ww (i,k,j) = 0.0
        uv (i,k,j) = 0.0;  vw (i,k,j) = 0.0;  wu (i,k,j) = 0.0
        fub(i,k,j) = 0.0;  fvb(i,k,j) = 0.0;  fwb(i,k,j) = 0.0
        ftb(i,k,j) = 0.0;  fqb(i,k,j) = 0.0;  wsb(i,k,j) = 0.0
        fab(i,k,j) = 0.0
      end do
      end do
      end do
c
      return
      end
