c **********************************************************************
c *   Surface fluxes (computed from the prescribed heat flux)          *
c *                                                                    *
c *   Input variables:                                                 *
c *     isfc          : = 1,       for horizontal-mean velocity        *
c *                   : > 1,       for velocities in all grids         *
c *     new           : = 0,       in the first call                   *
c *     z0m           : Roughness length for momentum             (m)  *
c *                   : If z0m = 0, the surface is the sea surface.    *
c *     z0h           : Dummy (not used)                               *
c *     tfmx, qfmx    : Fluxes of heat and vapor at noon (K m/s, m/s)  *
c *                   : These are positive upward, and vanish at       *
c *                   : sunrise and sunset.                            *
c *     dhour         : Length of the daytime                  (hour)  *
c *                   : Sunrise is at 12-(dhour/2) LST.                *
c *                   : If dhour=<0, fluxes are fixed at tfmx & qfmx.  *
c *     chour         : Current time starting from 00 LST      (hour)  *
c *                                                                    *
c *   Output variables (horizontal average):                           *
c *     ust           : Friction velocity                       (m/s)  *
c *     duz           : Vertical shear at a height of dz     (s^[-1])  *
c *                   : These are used only in sub.sgs_model.          *
c *                                                                    *
c *   # Note that zeta has a muximum according to log((z+z0m)/z0m)     *
c *     and alpha, which is equal to a double root, since its root     *
c *     changes to a complex number with increasing stability.         *
c **********************************************************************
c
      subroutine  surface_prescript ( isfc,
     :                new, z0m, z0h, tfmx, qfmx, dhour, chour,
     :                ust, duz )
c
      include  "Declare.h"
c
      data  hpai/1.570796/
      data  wmin,zm2h,elmn/ 0.01, 7.40, 0.10/
      data  gmm, gmh, aph /16.0,16.0, 5.0/, pr /1.00/, vk /0.40/
cBu   data  gmm, gmh, aph /15.0, 9.0, 4.7/, pr /0.74/, vk /0.35/
c
      z1   = 0.5 *dz
      tv0  = 0.61*tref
      rsum = 1.0 /dble((nx-2)*(ny-2))
c
      if ( dhour .gt. 0.0 ) then
        tsus = -tfmx*cos( hpai*( chour-12.0 )/( 0.5*dhour ) )
        qsus = -qfmx*cos( hpai*( chour-12.0 )/( 0.5*dhour ) )
      else
        tsus = -tfmx
        qsus = -qfmx
      end if
c
      wuav = 0.0
      wvav = 0.0
c
      do j = 2,ny-1
      do i = 2,nx-1
        wuav = wuav +u (i,1,j)
        wvav = wvav +v (i,1,j)
      end do
      end do
c
      wuav = wuav * rsum
      wvav = wvav * rsum
      ws1  = max( sqrt( wuav**2+wvav**2 ), wmin )
c
c  ***  ust over the sea surface  ***
      if ( new .eq. 0 ) then
        ust = ws1 * 0.05
        a   = log( z1*g/0.016    )
        b   = log( z1  /0.165e-5 )
c
        do iter = 1,5
          if ( ust .gt. 0.1 ) then
            f  = ust*( a-2.0*log( ust ) ) -vk*ws1
            df = a -2.0 -2.0*log( ust )
          else
            f  = ust*( b+    log( ust ) ) -vk*ws1
            df = b +1.0 +    log( ust )
          end if
c
          ust = ust -f/df
        end do
      end if
c
c  ***  Land surface  ***
      if ( z0m .gt. 0.0 ) then
                            z0mm = z0m
cx                          z0hh = z0m/zm2h
cx      if ( z0h .gt. 0.0 ) z0hh = z0h
c
c  ***  Sea surface  ***
      else
        if ( ust .gt. 0.1 ) then
                            z0mm = 0.016   *ust**2/g
        else
                            z0mm = 0.165e-5/ust
        end if
cx                          z0hh = 5.0e-5
      end if
c
      zz  = z1 +z0mm
      alm = log( zz/z0mm )
cx    alh = log( zz/z0hh ) * pr
      zz0 =      z0mm  /zz
      zzw = ( dz+z0mm )/zz
c
      sbv = g * zz                     / ( tref * vk**2 * ws1**3 )
      sbo = g * zz * ( tsus+tv0*qsus ) / ( tref * vk**2 )
      sb  = sbo / ws1**3
c
      call psi_phi ( sb, alm, zz0, zzw, gmm, gmh, aph, pr,
     :               psm, phm, phh, phw,  sbv, tvus )
c
      ust = vk *ws1/  psm
      duz = ust*phw/( vk*( dz+z0mm ) )
c
      if ( tvus .gt. 0.0 ) then
        if ( tsus .eq. 0.0 ) then
          qsus = tvus /  tv0
        else
          t2q  = qsus /  tsus
          tsus = tvus /( tv0*t2q+1.0 )
          qsus = tsus *  t2q
        end if
      end if
c
      if ( isfc .eq. 1 ) then
        do j = 2,ny-1
        do i = 2,nx-1
          flm(i,j) = -ust**2/ws1
          flt(i,j) = -tsus
          flq(i,j) = -qsus
          grm(i,j) =  phm / ( vk*zz*ust )
          grh(i,j) =  phh / ( vk*zz*ust )
        end do
        end do
c
      else
        do j = 2,ny-1
        do i = 2,nx-1
          ws1 = max( sqrt( u (i,1,j)**2+v (i,1,j)**2 ), wmin )
          sb  = sbo / ws1**3
c
          call psi_phi ( sb, alm, zz0, zzw, gmm, gmh, aph, pr,
     :               psm, phm, phh, phw,  sbv, tvus )
c
          us1 = vk *ws1/psm
c
          flm(i,j) = -us1**2/ws1
          flt(i,j) = -tsus
          flq(i,j) = -qsus
          grm(i,j) =  phm / ( vk*zz*us1 )
          grh(i,j) =  phh / ( vk*zz*us1 )
        end do
        end do
c
      end if
c
      return
      end
c
      subroutine  psi_phi ( sb, alm, zz0, zzw, gmm, gmh, aph, pr,
     :                      psm, phm, phh, phw,  sbv, tvus )
c
      tvus = -99.0
c
      if     ( sb .eq. 0.0 ) then
        psm = alm
        phm = 1.0
        phh = pr
        phw = 1.0
c
      elseif ( sb .gt. 0.0 ) then
        aa  =     sb*(aph*(1.0-zz0))**3
        bb  = 3.0*sb*(aph*(1.0-zz0))**2 * alm
        cc  = 3.0*sb* aph*(1.0-zz0)     * alm**2 -1.0
        dd  =     sb                    * alm**3
        pp  = cc/3.0 -bb**2/(  9.0*aa    )
        qq  = dd     +bb**3/( 13.5*aa**2 ) -bb*cc/( 3.0*aa )
        rr  = qq**2  +4.0*pp**3/aa
c
        if ( rr .ge. 0.0 ) then
          zet = 0.5 * alm /( aph*( 1.0-zz0 ) )
c
          psm = alm + aph*zet*( 1.0-zz0 )
          tvus= zet / ( sbv*psm**3 )
        else
          br  =       -qq  /( 2.0*aa )
          bi  = sqrt( -rr )/( 2.0*aa )
c
c  ***  The following compute the complex cube root of  ***
c  ***  br + i*bi, and obtain only its real part.       ***
c
          bth =  atan2( bi, br ) / 3.0
          brr = ( br**2 + bi**2 )**( 1.0/6.0 )
          bxx =  brr * cos( bth )
          byy =  brr * sin( bth )
          zet = -bxx + 1.732051*byy - bb/( 3.0*aa )
c
          psm = alm + aph*zet*( 1.0-zz0 )
        end if
c
        phm = 1.0 + aph*zet
        phh = pr  + aph*zet
        phw = 1.0 + aph*zet*zzw
c
      else
        zet = sb * alm**3
c
        do iter = 1,5
          rhm = ( 1.0-gmm*zet     )**0.25
          r0m = ( 1.0-gmm*zet*zz0 )**0.25
c
          psm = alm -log( ( rhm+1.0 )**2*( rhm**2+1.0 )
     :                 /( ( r0m+1.0 )**2*( r0m**2+1.0 ) ) )
     :         +2.0*( atan( rhm )-atan( r0m ) )
c
          f   = sb*psm**3 -zet
          df  = sb*psm**2 *( 3.0/rhm -3.0/r0m )/zet -1.0
          zet = zet -f/df
c
          if ( zet .ge. 0.0  ) zet = -1.0e-10
        end do
c
        phm = 1.0/    ( 1.0-gmm*zet     )**0.25
        phh = pr /sqrt( 1.0-gmh*zet     )
        phw = 1.0/    ( 1.0-gmm*zet*zzw )**0.25
c
      end if
c
      return
      end
