c **********************************************************************
c *   Solar and Longwave radiation (Katayama's scheme)                 *
c *                                                                    *
c *   Output variables:                                                *
c *     rhr(:, 1 ,:)  : Net downward radiative flux           (W/m^2)  *
c *     rhr(:,k+1,:)  : Heating rate at the k-th grid           (K/s)  *
c *     rfx           : Net downward longwave radiative flux  (W/m^2)  *
c *                                                                    *
c *   Work array:                                                      *
c *     uu            : Black-body radiative flux             (W/m^2)  *
c *     vv            : Effective water-vapor amount (EWVA)  (g/cm^2)  *
c *     ww            : Liquid-water path (LWP)              (g/ m^2)  *
c *     uv            : Transmission function                          *
c *     vw            : 1 - effective cloud amount (downward)          *
c *     wu            : 1 - effective cloud amount (upward)            *
c **********************************************************************
c
      subroutine  ewva_over_domain ( valp, pcrit, vvt2c, vvc2i )
c
      include  "Declare.h"
c
      ptop = 0.0
c
      do j = 2,ny-1
      do i = 2,nx-1
        ptop = ptop + pi0( i ,nz+1, j )
      end do
      end do
c
      ptop = ptop / dble((nx-2)*(ny-2))
      ptop = p0 * ( ptop/cp )**( cp/rd )
      qtop = 0.5* ( qi(nz+1)+qi( nz ) )
c
      q100 = 2.5e-6
      qfc  = log( q100/qtop ) / log( 100.0/ptop )
cx    qfc  = 3.65
      q000 = qtop*( p0   /ptop )**qfc
      q100 = qtop*( 100.0/ptop )**qfc
c
      vvt2c = 10.0*p0*q000*( ( ptop /p0 )**( valp+1.0+qfc )
     :                      -( pcrit/p0 )**( valp+1.0+qfc ) )
     :                    /( g * ( valp+1.0+qfc ) )
      vvc2i = 10.0*p0*q000*( ( pcrit/p0 )**( valp+1.0+qfc )
     :                      -( 100.0/p0 )**( valp+1.0+qfc ) )
     :                    /( g * ( valp+1.0+qfc ) )
     :       +10.0*p0*q100*  ( 100.0/p0 )**( valp+1.0     )
     :                    /( g * ( valp+1.0     ) )
c
      write(6,*) 'ptop,qtop,qfc=',ptop,qtop,qfc
      write(6,*) 'vvt2c,vvc2i=',vvt2c,vvc2i
c
      return
      end
c
      subroutine  avelm ( t, p, u, tav, pav, uav )
c
      include  "lesparm"
      dimension  t(mx,my,mz), p(mx,my,mz), u(mx,my,mz)
      dimension  tav(mz), pav(mz), uav(mz)
      common  /nxy/ nx, ny, nz, nxp1, nyp1, nzp1, nxm1, nym1, nzm1
c
      do 20 k = 1,nzp1
       tavk = 0.0
       pavk = 0.0
       uavk = 0.0
c
       do 10 j = 2,nym1
        do 10 i = 2,nxm1
         tavk = tavk +t(i,j,k)
         pavk = pavk +p(i,j,k)
         uavk = uavk +u(i,j,k)
   10  continue
c
       tav(k) = tavk/float((nx-2)*(ny-2))
       pav(k) = pavk/float((nx-2)*(ny-2))
       uav(k) = uavk/float((nx-2)*(ny-2))
   20 continue
c
      return
      end
c
      subroutine  radiation ( chour, rlat, elat, albd, clov, ulov )
c
      include  "Declare.h"
c
      dimension  tav(mz), pav(mz), uav(mz)
c
      dimension  vvx(mx,my), clx(mx,my), rem(mx,my), trm(mx,my),
     :           avu(mx,my), avd(mx,my)
      dimension  tau(mz+1,mz+1)
!hpf$ distribute (*,block) :: vvx, clx, rem, trm, avu, avd
c
      data  new / 0 /
c
      save  new
c
      interface
      end interface
c
c  *******************  Internal function  *******************
      abx( x ) = 1.0 -6.0*x/( (1.0+141.5*x)**0.635+5.925*x )
      acu( x ) = 1.0 -exp( -0.158*x )
      acd( x ) = 1.0 -exp( -0.130*x )
c  ***********************************************************
c  ***********************  Constants  ***********************
      ei0   = 1365.0
      sgm   =    5.67e-8
      valp  =    0.6
      scrit = sgm*220.0**4
      pcrit =  200.0        ! Rough pressure where T=220 K
      cszmn =    0.00022
c  ***********************************************************
c
      if ( new .eq. 0 ) then
        new = 1
        call ewva_over_domain ( valp, pcrit, vvt2c, vvc2i )
      end if
c
      do j = 1,ny
      do k = 1,nz+1
      do i = 1,mx
        rhr(i,k,j) = 0.0
        rfx(i,k,j) = 0.0
      end do
      end do
      end do
c
c  ***  The coefficient 0.62e-4 is based on Stephens et al. (1978)  ***
      do j = 2,ny-1
      do i = 2,nx-1
        vv (i,nz+1,j) = vvt2c -0.62e-4*clov*ulov
        ww (i,nz+1,j) = ulov
        cld(i,nz+1,j) = clov
      end do
      end do
c
      do j = 2,ny-1
      do k = nz,1,-1
      do i = 2,nx-1
        dp = 10.0 * ( p(i,k,j)-p(i,k+1,j) ) / g
c
        vv (i,k,j) = vv(ij,k+1) +dp*max( q(ij,k)-ql(ij,k),0.0 )
     :              *( 0.5*( p(ij,k)+p(ij,k+1) )/p0 )**valp
        ww (i,k,j) = ww(ij,k+1) +dp*1.0e4* ql(ij,k)
        cel(i,k,j) = cld(ij,k)
      end do
      end do
      end do
c
      call avelm ( t, p, vv, tav, pav, uav )
c
      d2r  = 3.141593/180.0
      clat = cos( rlat*d2r ) * cos( elat*d2r )
      slat = sin( rlat*d2r ) * sin( elat*d2r )
c
      omg = ( chour-12.0 )*3.141593/12.0
      cso = cos(omg)
      csz = slat +clat*cso
      if ( csz .lt. cszmn ) goto 100
c
      bsz = 35.0/sqrt( 1224.0*csz**2+1.0 )
      vvb = vvc2i * bsz
c
c  **  Solar( Short-wave ) radiation  **
      ss  = 0.517*ei0 * csz
      sa  = 0.483*ei0 * csz
c
c    **  Absorbed part  **
      call reflect ( dul, re,tr, mx,nx,ny, csz, 2 )
c
      do j = 2,ny-1
      do i = 2,nx-1
        clx(i,j) = 1.0 -clov
        rem(i,j) =      clov*      re(i,j)
        trm(i,j) = 1.0 -clov*( 1.0-tr(i,j) )
        vvx(i,j) = vvb +uav(nz+1)
     :                *( ( 1.0-clov )*bsz+1.66*clov )
      end do
      end do
c
      do 212 k = nz,1,-1
       kp1 = k +1
c
       do 211 ij = 1,mxy
       call reflect ( ww(i,k,j), re,tr, mx,nx,ny, csz, 2 )
c
         clx(ij) = clx(ij)*( 1.0-cel(ij,k) )
         cle     =           1.0-clx(ij)
         vvx(ij) = vvx(ij) +( uav(k)-uav(kp1) )
     :                     *( ( 1.0-cle )*bsz+1.66*cle )
c
         rec     = ( re(ij)-rem(ij) )
     :          /( ( re(ij)-rem(ij) )*rem(ij)+trm(ij)**2 )
         trre    = trm(ij)/( 1.0-rec*rem(ij) )
         uv(ij,kp1) = amin1( 1.0-rec*trre,1.0 )
c
         rem(ij) = re(ij)
         trm(ij) = tr(ij)
         rhr(ij,k) = amax1( sa*abx( vvx(ij) )*trm(ij),0.0 )
  211  continue
  212 continue
c
      do j = 2,ny-1
      do k = nz,2,-1
      do i = 2,nx-1
        rhr(i,k,j) = max( rhr(i,k,j)*uv(i,k,j)-rhr(i,k-1,j), 0.0 )
      end do
      end do
      end do
c
      do j = 2,ny-1
      do i = 2,nx-1
        rhr(i,1,j) = rhr(i,1,j)/( 1.0-rem(i,j)*albd )
      end do
      end do
c
c    **  Scattered part ( Ground surface )  **
      call reflect ( ww(1,1), rem,trm, mx,nx,ny, csz, 1 )
c
      do 223 ij = 1,mxy
         alb0 = amin1( 0.107-0.311*alog10(pav(1)/p0*csz),1.0 )
         alb0 = 1.0 -( 1.0-alb0 )*trm(ij)
         alb1 = 1.0 -  0.8675    *trm(ij)
c
         sss  = ss *( 1.0-alb0 )/( 1.0-alb1*alb(ij) )
         rhr(ij,1) = rhr(ij,1) +sss
  223 continue
c
  100 continue
c
      do 280 ij = 1,mxy
  280   sun(ij) = ( 1.0-alb(ij) )*rhr(ij,1)
c
c  **  Black body radiation  **
      do j = 2,ny-1
      do i = 2,nx-1
        p2a = 0.5*( pi(i,2,j)+pi0(i,2,j)
     :             +pi(i,1,j)+pi0(i,1,j) )
        tt  = p2a*( t (i,1,j)+tref ) + ev * ql(i,1,j)
        ta  = 0.5*( tt/cp + ts(i,j)+tref )
        uu (i,1,j) = sgm*  ta**4
        vvx(i,  j) = sgm*( ta**4-( ts(i,j)+tref )**4 )
      end do
      end do
c
      do j = 2,ny-1
      do k = 2,nz+1
      do i = 1,mx
        p2a = pi(i,k,j) +pi0(i,k,j)
        tt  = p2a*( t (i,k,j)+t ( i ,k-1, j )+2.0*tref )
     :       +ev *( ql(i,k,j)+ql( i ,k-1, j )          )
        uu (i,k,j) = sgm*( 0.5*tt/cp )**4
      end do
      end do
      end do
c
c  ***  Mean transmission function (tac)  ***
      do k = 1,nz
        if ( uav(k) .lt. 0.0001 ) then
          tva = 1.0/( 1.0+2.56*uav(k)**0.39 )
        else
          zet = log10( uav(k) )
          if ( zet .gt. -1.0 ) then
            tva = 0.254 -( 0.1985-0.0205*zet )*zet
          else
            tva = 0.216 -( 0.2827+0.0258*zet )*zet
          end if
        end if
c
        uco2 = ( pav(k)*pav(k)-pcrit*pcrit )/( p0*p0 )
        tco2 =  0.791 -0.066*log10(uco2)
c
        do j = 2,ny-1
        do i = 2,nx-1
          uv(i,k,j) = tco2 * tva
        end do
        end do
c
      end do
c
c  **  1 - Effective cloud amount  **
      do 345 k = 1,nz
       do 345 ij = 1,mxy
         if ( cld(ij,k) .gt. 0.0 ) then
           ceu(ij,k) = 1.0 -cld(ij,k)*acu( ww(ij,k)-ww(ij,k+1) )
           cel(ij,k) = 1.0 -cld(ij,k)*acd( ww(ij,k)-ww(ij,k+1) )
         else
           ceu(ij,k) = 1.0
           cel(ij,k) = 1.0
         end if
  345 continue
c
c  **  Mean transmission function ( tau )  **
      do 360 l = 1,nz
         tau(l,l) = 1.0
c
         do 350 k = l+1,nzp2
          if ( k .eq. nzp2 ) then
c?          uaa = uav(l) -0.5*uav(nzp1)
c?          pp  = 0.5*( pcrit+pav(nzp1) )
            uaa = uav(l)
            pp  = pcrit
          else
            uaa = uav(l) -uav(k)
            pp  = pav(k)
          end if
c
          if ( uaa .lt. 0.0001 ) then
            tva = 1.0/( 1.0+298.7*uaa )
          else
            zet = alog10( uaa )
            if ( zet .gt.  0.0 ) then
              tva = 0.373 -( 0.274 -0.035 *zet )*zet
            else
              tva = 0.373 -( 0.2595+0.0275*zet )*zet
            end if
          end if
c
          uco2 = amax1( (pav(l)*pav(l)-pp*pp)/(p0*p0),1.0e-10 )
          tco2 =  0.791 -0.066*alog10(uco2)
          if ( tco2 .gt. 1.0 ) tco2 = 1.0
c
          tau(l,k) = tco2*tva
          tau(k,l) = tco2*tva
  350    continue
  360 continue
c
c  **  Long-wave radiation  **
      do 420 k = 1,nz
         km1 = max0( k-1,1 )
c
         do 410 ij = 1,mxy
          qm  = 500.0*( q(ij,km1)-ql(ij,km1)+q(ij,k)-ql(ij,k) )
          if ( qm .lt. 0.1 ) qm = 0.1
cx !CDIR IEXPAND
cx        call avfac( p(ij,k),qm,p(ij, k )-p(ij,k+1),
cx   :                           p(ij,km1)-p(ij, k ),
cx   :                avu(ij),avd(ij) )
c
c *** avfac ***
          pm  = p(ij, k )
          dpu = p(ij, k ) -p(ij,k+1)
          dpl = p(ij,km1) -p(ij, k )
c
      zi  = alog10( qm )
      z1  = abs( zi+2.5 )
      ap  = alog10( pm )
c
      al  = -1.66  + 1.76  *ap
      bl  = -0.197 + 0.0002*pm
      af  =  0.3   *zi + 0.28 *zi**2 + 0.04   *zi**3
      bf  =  0.0812*z1 - 0.045*z1**2 + 0.02334*z1**3
      dk  = -0.65*max( 0.041  -0.021*zi, 0.06   )
      dg  = -4.0 *max( 0.01225+0.007*zi, 0.0093 )
      aip =  al + af
      bip =  bl + bf + dk + dg
c
      al  =  max( 61.86-22.92*ap, 76.63-28.39*ap )
      bl  = -max( 42.59-15.78*ap, 60.81-22.53*ap )
      z1  =  zi - 0.105*al
      af  =  2.57 + 0.233*z1 + 0.18*z1**2 + 0.027*z1**3
      z1  =  zi - 0.105*bl
      bf  =  1.42 + 0.48 *z1 + 0.16*z1**2 + 0.011*z1**3
      dk  =  0.65*( 0.08 + (0.371-0.102*ap)*(zi+2.1) )
      dg  =  4.0 *max( 0.0325+0.005*zi, 0.0275 )
      aim = -0.09*al + af
      bim = -0.09*bl + bf + dk + dg
c
      avd(ij) = aip + bip*dpl/100.0
      avu(ij) = aim + bim*dpu/100.0
c *** avfac ***
  410    continue
c
         do 411 ij = 1,mxy
  411     clx(ij) = 1.0
c
       if ( k .gt. 1 ) then
         l = k-1
         do 412 ij = 1,mxy
          clx(ij) = clx(ij)*cel(ij,l)
          tau2    = (1.0+avd(ij)*tau( l ,k)*clx(ij))/(1.0+avd(ij))
          rfx(ij,k) = rfx(ij,k) +tau2*( uu(ij,l+1)-uu(ij,l) )
  412    continue
         do 413 l = k-2,1,-1
         do 413 ij = 1,mxy
          trm(ij) = clx(ij)
          clx(ij) = clx(ij)*cel(ij,l)
          tau2    = 0.5*( tau(l,k)*clx(ij)+tau(l+1,k)*trm(ij) )
          rfx(ij,k) = rfx(ij,k) +tau2*( uu(ij,l+1)-uu(ij,l) )
  413    continue
       end if
c
         do 414 ij = 1,mxy
  414     rfx(ij,k) = rfx(ij,k) +tau(1,k)*vvx(ij) *clx(ij)
c
         do 416 ij = 1,mxy
  416     clx(ij) = 1.0
c
         l = k
         do 417 ij = 1,mxy
          clx(ij) = clx(ij)*ceu(ij,l)
          tau2    = (1.0+avu(ij)*tau(l+1,k)*clx(ij))/(1.0+avu(ij))
          rfx(ij,k) = rfx(ij,k) +tau2*( uu(ij,l+1)-uu(ij,l) )
  417    continue
         do 418 l = k+1,nz
         do 418 ij = 1,mxy
          trm(ij) = clx(ij)
          clx(ij) = clx(ij)*ceu(ij,l)
          tau2    = 0.5*( tau(l,k)*trm(ij)+tau(l+1,k)*clx(ij) )
          rfx(ij,k) = rfx(ij,k) +tau2*( uu(ij,l+1)-uu(ij,l) )
  418    continue
c
         do 419 ij = 1,mxy
cx        cldp1 = 1.0 -cup(ij)*acu( dul(ij) )
          cldp1 = 1.0
          cly   = clx(ij)*cldp1
          rfx(ij,k) = rfx(ij,k) -cly*( scrit*uv(ij,k)
     :               +( uu(ij,nzp1)-scrit )*tau(nzp2,k) )
  419    continue
  420 continue
c
c  **  Cooling rate  **
      do 510 k = 2,nz
       do 510 ij = 1,mxy
         pp = 50.0*( pi(ij, k )+pi0(ij, k )
     :              +pi(ij,k-1)+pi0(ij,k-1) )
     :            *( p(ij,k-1)-p(ij,k) )
         rhr(ij,k) = g*( rfx(ij,k)-rfx(ij,k-1)
     :                  +rhr(ij,k)              )/pp
  510 continue
c
      do j = 2,ny-1
      do i = 2,nx-1
        rsun = ( 1.0-albd )*rhr(i,1,j)
        rhr(i,  1 ,j) = rfx(i, 1,j) + rsun
        rhr(i,nz+1,j) = rhr(i,nz,j)
      end do
      end do
c
      rsum = 1.0 /float((nx-2)*(ny-2))
c
      do k = 1,nz
        rhrk = 0.0
c
      do j = 2,ny-1
      do i = 2,nx-1
        rhrk = rhrk +rhr(i,k,j)
      end do
      end do
c
        ti(k) = ti(k) + rhrk*rsum * dt
      end do
c
      return
      end
c
      pure subroutine  reflect ( ul, re, tr, mx, nx, ny, csz, m )
c
c    @ Reflection & Transmission of Solar radiation through clouds @
c                                      after Stephens(1978,1984)
c      m =  1 : Scattered part
c           2 : Absorbed  part
c
c      ul     : Liquid water amount ( g/m2 )
c      beta   : Backscatter function
c      omeg   : 1 - ( Single scattering albedo )
c
      dimension  ul(mx,ny),  re(mx,ny), tr(mx,ny)
      dimension  bt1(10), bt2(15), om(15)
      data     bt1/ 0.1145  ,-0.05375 , 0.1024  , 0.006667, 0.066035,
     :             -0.3251  ,-1.222e-4,-0.007468, 0.005461, 0.1414  /
      data     bt2/ 0.09659 ,-0.07529 , 0.3812  , 0.02057 , 0.05642 ,
     :             -1.082   ,-0.002786,-0.02675 , 0.1298  , 0.8616  ,
     :              1.724e-4, 0.001225, 0.004154,-0.08173 ,-0.2185  /
      data     om /-0.003022, 3.050e-4, 0.04591 , 2.483e-4,-0.02182 ,
     :              0.02216 , 3.536e-5, 4.974e-4, 0.02128 ,-0.071655,
     :             -1.149e-5, 2.656e-4,-0.002867, 0.001395, 0.02856 /
c
      csz2 = amax1( csz,0.1  )
c?    csz2 = amax1( csz,0.25 )
      scz  = 1.0/csz2
c
      if ( m .eq. 1 ) then
        bcz0 = bt1(1) +( bt1(3)+bt1(6)*csz2+bt1(10)*csz2**2 )*csz2
        bcz1 = bt1(2) +( bt1(5)+bt1(9)*csz2                 )*csz2
        bcz2 = bt1(4) +  bt1(8)*csz2
        bcz3 = bt1(7)
c
        do 120 j = 1,ny
         do 120 i = 1,mx
          if ( ul(i,j) .lt. 1.1 ) then
            re(i,j) = 0.0
            tr(i,j) = 1.0
c
          else
            algt = 0.6063 +3.9363*alog( alog10( ul(i,j) ) )
            tau  = exp( algt )
c
cx          n    = 0
cx          beta = 0.0
cx          do 110 k = 1,4
cx           do 110 l = 1,k
cx            n    = n +1
cx110         beta = beta +bt1(n) * algt**( k-l ) * csz2**( l-1 )
c
            beta = bcz0 +( bcz1+bcz2*algt+bcz3*algt**2 )*algt
            if ( beta .lt. 0.0    ) beta = 0.0
c
            tr(i,j) = 1.0/( 1.0+beta*tau*scz )
            re(i,j) = 1.0 -tr(i,j)
c
          end if
  120   continue
c
      else
        bcz0 = bt2(1) +( bt2(3)+bt2(6)*csz2+bt2(10)*csz2**2
     :                                     +bt2(15)*csz2**3 )*csz2
        bcz1 = bt2(2) +( bt2(5)+bt2(9)*csz2+bt2(14)*csz2**2 )*csz2
        bcz2 = bt2(4) +( bt2(8)+bt2(13)*csz2                )*csz2
        bcz3 = bt2(7) +  bt2(12)*csz2
        bcz4 = bt2(11)
c
        ocz0 = om(1) +( om(3)+om(6)*csz2+om(10)*csz2**2
     :                                  +om(15)*csz2**3 )*csz2
        ocz1 = om(2) +( om(5)+om(9)*csz2+om(14)*csz2**2 )*csz2
        ocz2 = om(4) +( om(8)+om(13)*csz2               )*csz2
        ocz3 = om(7) +  om(12)*csz2
        ocz4 = om(11)
c
        do 220 j = 1,ny
         do 220 i = 1,mx
          if ( ul(i,j) .lt. 1.1 ) then
            re(i,j) = 0.0
            tr(i,j) = 1.0
c
          else
            algt = 0.8041 +3.8034*alog( alog10( ul(i,j) ) )
            tau  = exp( algt )
c
cx          n    = 0
cx          beta = 0.0
cx          omeg = 0.0
cx          do 210 k = 1,5
cx           do 210 l = 1,k
cx            n    = n +1
cx            x    = algt**( k-l ) * csz2**( l-1 )
cx            beta = beta +bt2(n) * x
cx210         omeg = omeg +om(n)  * x
c
            beta = bcz0 +( bcz1+bcz2*algt+bcz3*algt**2
     :                                   +bcz4*algt**3 )*algt
            omeg = ocz0 +( ocz1+ocz2*algt+ocz3*algt**2
     :                                   +ocz4*algt**3 )*algt
            if ( beta .lt. 0.0    ) beta = 0.0
            if ( omeg .lt. 1.0e-4 ) omeg = 1.0e-4
c
            u  =  sqrt( ( omeg+2.0*beta*(1.0-omeg) )/omeg )
            et =  exp( -u*omeg*tau*scz )
            r  = ( u+1.0 )**2 -( et*( u-1.0 ) )**2
c
            tr(i,j) =  4.0*u*et/r
            re(i,j) = ( u*u-1.0 )*( 1.0-et*et )/r
c
          end if
  220   continue
c
      end if
c
      return
      end
c
      subroutine  avfac( p, q, dpu, dpl, avu, avd )
c
c     p,dp : hPa
c     q    : g/kg
c
      zi  = alog10( q )
      z1  = abs( zi+2.5 )
      ap  = alog10( p )
c
      al  = -1.66  + 1.76  *ap
      bl  = -0.197 + 0.0002*p
      af  =  0.3   *zi + 0.28 *zi**2 + 0.04   *zi**3
      bf  =  0.0812*z1 - 0.045*z1**2 + 0.02334*z1**3
      dk  = -0.65*max( 0.041  -0.021*zi, 0.06   )
      dg  = -4.0 *max( 0.01225+0.007*zi, 0.0093 )
      aip =  al + af
      bip =  bl + bf + dk + dg
c
      al  =  max( 61.86-22.92*ap, 76.63-28.39*ap )
      bl  = -max( 42.59-15.78*ap, 60.81-22.53*ap )
      z1  =  zi - 0.105*al
      af  =  2.57 + 0.233*z1 + 0.18*z1**2 + 0.027*z1**3
      z1  =  zi - 0.105*bl
      bf  =  1.42 + 0.48 *z1 + 0.16*z1**2 + 0.011*z1**3
      dk  =  0.65*( 0.08 + (0.371-0.102*ap)*(zi+2.1) )
      dg  =  4.0 *max( 0.0325+0.005*zi, 0.0275 )
      aim = -0.09*al + af
      bim = -0.09*bl + bf + dk + dg
c
      avd = aip + bip*dpl/100.0
      avu = aim + bim*dpu/100.0
c
      return
      end
