c **********************************************************************
c *   Surface fluxes (computed from the force restore method           *
c *                            with the Euler forward scheme)          *
c *                                                                    *
c *   Input variables:                                                 *
c *     new           : = 0,       in the first call                   *
c *     z0m,  z0h     : Roughness length for momentum and heat    (m)  *
c *                   : If z0m = 0, the surface is the sea surface.    *
c *                   : If z0h = 0, z0h is set to z0m/7.4.             *
c *                   : However, z0h on the sea is fixed at 5.0e-5.    *
c *     tdeep         : Deep soil temperature                     (K)  *
c *     swet          : Soil wetness                        (0.0-1.0)  *
c *     crso          : Soil heat capacity                  (J/K m^3)  *
c *     fkso          : Soil thermal conductivity             (W/K m)  *
c *                   : If either crso or fkso is less than zero,      *
c *                   : both of them are computed from swet.           *
c *                                                                    *
c *   Output variables (horizontal average):                           *
c *     ust           : Friction velocity                       (m/s)  *
c *     duz           : Vertical shear at a height of dz     (s^[-1])  *
c *                   : These are used only in sub.sgs_model.          *
c **********************************************************************
c
      subroutine  surface_forecast (
     :                new, z0m, z0h, tdeep, swet, crso, fkso,
     :                ust, duz )
c
      include  "Declare.h"
c
      data  wmin,zm2h,elmn/ 0.01, 7.40, 0.10/
c
      z1   = 0.5 *dz
      rsum = 1.0 /dble((nx-2)*(ny-2))
c
                          bet = 1.0
      if ( z0m .gt. 0.0 ) bet = min( 2.0*swet, 1.0 )
c
      wuav = 0.0
      wvav = 0.0
      dtav = 0.0
c
!hpf$ independent, new(i), reduction(wuav, wvav, dtav)
      do j = 2,ny-1
      do i = 2,nx-1
        a2p  = cp / ( pi(i,1,j)+pi0(i,1,j) )
        tss  = ts(i,j) +tref -t0
        qss  = 3.8*exp( 17.27*tss/( 237.3+tss ) ) / p (i,1,j)
c
          vv(i,1,j) = t (i,1,j) +tref -a2p*( tss+t0 )
c
        if ( qss .gt. q (i,1,j)-ql(i,1,j) ) then
          ww(i,1,j) = bet*( q (i,1,j)-ql(i,1,j)-qss ) +ql(i,1,j)
        else
          ww(i,1,j) =       q (i,1,j)          -qss
        end if
c
        wuav = wuav +  u (i,1,j)
        wvav = wvav +  v (i,1,j)
        dtav = dtav +( vt(i,1,j) +0.61*  q (i,1,j) +1.0    )
     :              *  vv(i,1,j)
     :              +( vq(i,1,j) +0.61*( t (i,1,j) +tref ) )
     :              *  ww(i,1,j)
      end do
      end do
c
      wuav = wuav * rsum
      wvav = wvav * rsum
      dtav = dtav * rsum
      ws1  = max( sqrt( wuav**2+wvav**2 ), wmin )
c
c  ***  ust over the sea surface  ***
      if ( new .eq. 0 ) then
        ust = ws1 * 0.05
        a   = log( z1*g/0.016    )
        b   = log( z1  /0.165e-5 )
c
        do iter = 1,5
          if ( ust .gt. 0.1 ) then
            f  = ust*( a-2.0*log( ust ) ) -vk*ws1
            df = a -2.0 -2.0*log( ust )
          else
            f  = ust*( b+    log( ust ) ) -vk*ws1
            df = b +1.0 +    log( ust )
          end if
c
          ust = ust -f/df
        end do
      end if
c
c  ***  Land surface  ***
      if ( z0m .gt. 0.0 ) then
                            z0mm = z0m
                            z0hh = z0m/zm2h
        if ( z0h .gt. 0.0 ) z0hh = z0h
c
        crw = crso
        fkw = fkso
        if ( crso .le. 0.0 .or. fkso .le. 0.0 ) then
          crw = 4.186e6*( 0.386 +  0.15*swet  )
          fkw = 4.186e2*  0.002 *( 1.00+swet )
        end if
        cf2 = 3.141593/( 12.0*3600.0 )
        cf1 = sqrt( 2.0*cf2/( crw*fkw ) )
c
c  ***  Sea surface  ***
      else
        if ( ust .gt. 0.1 ) then
                            z0mm = 0.016   *ust**2/g
        else
                            z0mm = 0.165e-5/ust
        end if
                            z0hh = 5.0e-5
c
        cf2 = 0.0
        cf1 = 0.0
      end if
c
      zz   = z1 +z0mm
      alm  = log( zz/z0mm )
      alh  = log( zz/z0hh ) * spr
      zz0m =      z0mm  /zz
      zz0h =      z0hh  /zz
      zzw  = ( dz+z0mm )/zz
c
      rb   = g * zz * dtav / ( tref * ws1**2 )
c
      if ( rb .ge. 0.0 ) then
        aa  = rb * ( aph*(1.0-zz0m) )**2  - aph*(1.0-zz0h)
        bb  = alh    - 2.0*alm * rb * aph*      (1.0-zz0m)
        rr  = alh**2 + 4.0*alm * rb * aph*( alm*(1.0-zz0h)
     :                                     -alh*(1.0-zz0m) )
c
        if ( aa .ge. 0.0 ) then
          zet = zz/elmn
        else
          zet = 0.5 * ( bb - sqrt( rr ) ) / aa
c
          if ( zet .gt. zz/elmn ) zet = zz/elmn
        end if
c
        psm = alm +aph*zet*( 1.0-zz0m )
        phm = 1.0 +aph*zet
        phh = spr +aph*zet
        phw = 1.0 +aph*zet*  zzw
c
        tst = vk /( alh+aph*zet*( 1.0-zz0h ) )
c
      else
        zet = rb * alm**2 / alh
c
        do iter = 1,4
          rhm =     ( 1.0-gmm*zet      )**0.25
          r0m =     ( 1.0-gmm*zet*zz0m )**0.25
          rhh = sqrt( 1.0-gmh*zet      )
          r0h = sqrt( 1.0-gmh*zet*zz0h )
c
          psm = alm -log( ( rhm+1.0 )**2*( rhm**2+1.0 )
     :                 /( ( r0m+1.0 )**2*( r0m**2+1.0 ) ) )
     :         +2.0*( atan( rhm )-atan( r0m ) )
          psh = alh -2.0*spr*log( ( rhh+1.0 )/( r0h+1.0 ) )
c
          pwk = psm / psh
          f   = rb * psm * pwk - zet
          df  = rb * ( 2.0    /rhm -2.0    /r0m
     :                -pwk*spr/rhh +pwk*spr/r0h )*pwk / zet - 1.0
          zet = zet -f/df
c
          if ( zet .ge. 0.0  ) zet = -1.0e-10
        end do
c
        phm = 1.0/    ( 1.0-gmm*zet      )**0.25
        phh = spr/sqrt( 1.0-gmh*zet      )
        phw = 1.0/    ( 1.0-gmm*zet*zzw  )**0.25
c
        tst = vk /psh
c
      end if
c
      ust = vk *ws1/  psm
      duz = ust*phw/( vk*( dz+z0mm ) )
c
      do j = 2,ny-1
      do i = 2,nx-1
        p2a  = ( pi(i,1,j)+pi0(i,1,j) ) / cp
c  ***  Strictly, p2a here is the average of pi at k=1 and 2.  ***
        tt   = p2a * ( t (i,1,j)+tref ) + ev/cp * ql(i,1,j)
        rho  = 100.0 * p (i,1,j) / ( rd * tt )
c
        tsus = tst*ust*vv(i,1,j)
        qsus = tst*ust*ww(i,1,j)
c
        flm(i,j) = -ust**2/ws1
        flt(i,j) = -tsus
        flq(i,j) = -qsus
        grm(i,j) =  phm / ( vk*zz*ust )
        grh(i,j) =  phh / ( vk*zz*ust )
c
        budg = rhr(i,1,j) + rho*( cp*tsus*p2a + ev*qsus )
        fso  = budg*cf1 - ( ts(i,j)+tref - tdeep )*cf2
c
        ts(i,j) = ts(i,j) + dt*fso
      end do
      end do
c
      call bc_lat_scalar2 ( flm )
c
      return
      end
